//=============================================================================
//  MuseScore
//  Music Composition & Notation
//
//  Brass Fingering Plugin
//
//  Copyright (C) 2012 Werner Schweer
//  Copyright (C) 2013, 2014 Joachim Schmitz
//  Copyright (C) 2014 Jörn Eichler
//  Copyright (C) 2015 minitubist
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//=============================================================================

import QtQuick 2.0
import MuseScore 1.0

MuseScore {
   version: "2.0"
      
   description: qsTr("Fingering for brass.\n") 
   +  qsTr("Brassfingering (also availible for MuseScore 1.3) shows fingering for french horn, tuba, trumpet or trombone as specified in Staff.Property  (Score.parts[0].instrumentID).\n")
   +  qsTr("If you change the instrument, remove the fingering (see below) and click Brassfingering from Plugins.\n")
   +  qsTr("It ist easy to change for other instruments.\n")
   +  qsTr("To remove fingering, click right on number, select all, delete.");menuPath: "Plugins.Brassfingering"
 
   property var griff1: ["123","13","23","12","1","2","0","123","13","23","12","1","2","0","23","12","1","2","0","12","1","2","0","1","2","0","23","12","1","2","0","12","1","2","0"];
   property var griff2: ["24","4","23","12","1","2","0","24","4","23","12","1","2","0","23","12","1","2","0","12","1","2","0","1","2","0","23","12","1","2","0","","","",""];
   property var griff3: ["7","6","5","4","3","2","1","7","6","5","4","3","2","1","5","4","3","2","1","4","3","2","1","3","2","1","3+","2+","3","2","1","","","",""];

   function fHorn(pitch){
     var s= griff1[pitch-40];
     if (typeof s ==="undefined") s="";
     return s;
     }

   function fTuba3(pitch){
     var s= griff1[pitch-28];
     if (typeof s ==="undefined") s="";
     return s;
     }

   function fTuba4(pitch){
     var s= griff2[pitch-28];
     if (typeof s ==="undefined") s="";
     return s;
     }

   function fTrompete(pitch){
     var s= griff1[pitch-52];
     if (typeof s ==="undefined") s="";
     return s;
     }

   function fPosaune(pitch){
     var s= griff3[pitch-42];
     if (typeof s ==="undefined") s="";
     return s;
     }

   function nameChord (instrument, notes, text, oldtext) {
   console.log( instrument);
   
for (var i = 0; i < notes.length; i++) {
         var sep = "\n"; // change to "," if you want them comma-separated
         if ( i > 0 )
            text.text = sep + text.text;
         if (typeof notes[i].tpc === "undefined") // just in case
            return
         switch(instrument) {
            case "brass.french-horn":   text.text = fHorn(notes[i].pitch) + text.text; break;
            //case "tuba":   text.text = fTuba3(notes[i].pitch) + text.text; break;
            case "brass.tuba":   text.text = fTuba4(notes[i].pitch) + text.text; break;
            case "brass.trumpet":   text.text = fTrompete(notes[i].pitch) + text.text; break;
            case "brass.trombone":   text.text = fPosaune(notes[i].pitch) + text.text; break;
            default: break;
            }
      } // end for note
      if (text.text == oldtext.text) text.text ="";
   }

   onRun: {
      if (typeof curScore === 'undefined')
        Qt.quit();
      var cursor = curScore.newCursor();
      var startStaff;
      var endStaff;
      var endTick;
      var fullScore = false;
      var oldtext="";
      cursor.rewind(1);
      if (!cursor.segment) { // no selection
        fullScore = true;
        startStaff = 0; // start with 1st staff
        endStaff  = curScore.nstaves - 1; // and end with last
      } else {
        startStaff = cursor.staffIdx;
        cursor.rewind(2);
        if (cursor.tick == 0) {
          // this happens when the selection includes
          // the last measure of the score.
          // rewind(2) goes behind the last segment (where
          // there's none) and sets tick=0
          endTick = curScore.lastSegment.tick + 1;
        } else {
          endTick = cursor.tick;
        }
        endStaff   = cursor.staffIdx;
      }
      console.log(startStaff + " - " + endStaff + " - " + endTick)

      for (var staff = startStaff; staff <= endStaff; staff++) {
        for (var voice = 0; voice < 4; voice++) {
          cursor.rewind(1); // beginning of selection
          cursor.voice    = voice;
          cursor.staffIdx = staff;

          if (fullScore)  // no selection
            cursor.rewind(0); // beginning of score
      var instrument = curScore.parts[0].instrumentId;

          while (cursor.segment && (fullScore || cursor.tick < endTick)) {
            if (cursor.element && cursor.element.type == Element.CHORD) {
              var text = newElement(Element.STAFF_TEXT);

              var graceChords = cursor.element.graceNotes;
              for (var i = 0; i < graceChords.length; i++) {
                // iterate through all grace chords
                var notes = graceChords[i].notes;
                nameChord(instrument, notes, text, oldtext);
                // there seems to be no way of knowing the exact horizontal pos.
                // of a grace note, so we have to guess:
                text.pos.x = -2.5 * (graceChords.length - i);
                switch (voice) {
                  case 0: text.pos.y =  1; break;
                  case 1: text.pos.y = 10; break;
                  case 2: text.pos.y = -1; break;
                  case 3: text.pos.y = 12; break;
                }

                cursor.add(text);
                // new text for next element
                if (text.text > "") oldtext=text;
                text  = newElement(Element.STAFF_TEXT);
              }

              var notes = cursor.element.notes;
              nameChord(instrument, notes, text, oldtext);

              switch (voice) {
                case 0: text.pos.y =  1; break;
                case 1: text.pos.y = 10; break;
                case 2: text.pos.y = -1; break;
                case 3: text.pos.y = 12; break;
              }
              if ((voice == 0) && (notes[0].pitch > 83))
                text.pos.x = 1;
              cursor.add(text);
              if (text.text > "") oldtext=text;
            } // end if CHORD
            cursor.next();
          } // end while segment
        } // end for voice
      } // end for staff
      Qt.quit();
   } // end onRun
}
